using System;

using System.Collections;
//using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Windows.Forms;



//using System.Windows.Controls.Canvas;
//using Syncfusion.Windows.Forms.Design;
using Syncfusion.Windows.Forms.Diagram;
using Syncfusion.Windows.Forms.Diagram.Controls;
//using Syncfusion.Windows.Forms.Tools;


namespace Symcon.LightRidge.DashBoardControlsLibrary.PropertyBag
{
	[
	ToolboxItem(true),
	Description("Property Editor control for nodes in a canvas.")
	]
	public class PropertyEditor : Panel
	{
		private System.ComponentModel.IContainer components;
        private Diagram canvas = null;
		private PropertyGrid			propGrid;
		/// <summary>
		/// Initializes a new instance of the <see cref="PropertyEditor"/> class.
		/// </summary>
		public PropertyEditor()
		{
			// This call is required by the Windows.Forms Form Designer.
			InitializeComponent();

			// Create embedded property grid and configure it.
			this.propGrid = new PropertyGrid();
			this.Controls.Add(this.propGrid);
			this.propGrid.Dock = DockStyle.Fill;
			this.propGrid.PropertyValueChanged += new PropertyValueChangedEventHandler(this.PropertyGrid_ValueChanged);
		}


		/// <summary>
		/// Selects the list of nodes in the property grid.
		/// </summary>
		/// <param name="nodes">Nodes to display in the property editor.</param>
		public void SetSelectedObjects(NodeCollection nodes)
		{
			if (nodes.Count > 0)
			{
				Type nodeType = nodes[0].GetType();
				this.propGrid.SelectedObject = nodeType.GetProperty("PT").GetValue(nodes[0], null);
			}
			else
			{
				this.propGrid.SelectedObject = null;
			}
		}

		/// <summary>
		/// Select the node in the property grid.
		/// </summary>
		/// <param name="node">Node to display in the property editor.</param>
		public void SetSelectedObject(INode node)
		{
			if(node != null)
			{
				Type nodeType = node.GetType();
				if(nodeType == typeof(Model))
					this.propGrid.SelectedObject = this.canvas.Model;
				else
				    this.propGrid.SelectedObject = nodeType.GetProperty("PT").GetValue(node, null);
			}
		}

		/// <summary>
		/// Reference to the PropertyGrid object contained by this property editor.
		/// </summary>
		public System.Windows.Forms.PropertyGrid PropertyGrid
		{
			get
			{
				return this.propGrid;
			}
		}

		/// <summary>
		/// canvas the property editor is attached to.
		/// </summary>
		/// <value>The canvas.</value>
		/// <remarks>
		/// This property contains a reference to the canvas that this property
		/// editor is attached to. The property editor receives events from the
		/// canvas when the current selection changes and it updates the currently
		/// displayed object in the property editor.
		/// </remarks>
		[
		Browsable(true),
		DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)
		]
		public Diagram Canvas
		{
			get
			{
				return this.canvas;
			}
			set
			{
				if (this.canvas != value)
				{
					if (this.canvas != null)
					{
                        this.canvas.EventSink.SelectionListChanged -= new CollectionExEventHandler(EventSink_SelectionListChanged);
                        this.canvas.Model.EventSink.PropertyChanged -= new Syncfusion.Windows.Forms.Diagram.PropertyChangedEventHandler(EventSink_PropertyChanged);
                        this.canvas.Model.EventSink.PinPointChanged -= new PinPointChangedEventHandler(EventSink_PinPointChanged);
                        this.canvas.Disposed += new EventHandler(canvas_Disposed);
                        
                        //this.canvas.SelectionChanged -= new NodeCollectionEventHandler(canvas_SelectionChanged);
                        //this.canvas.PropertyChanged -= new PropertyEventHandler(canvas_PropertyChanged);
						//this.canvas.NodeMoved -= new MoveEventHandler(canvas_NodeMoved);
					}

					this.canvas = value;

					if (this.canvas != null)
					{
                        this.canvas.EventSink.SelectionListChanged += new CollectionExEventHandler(EventSink_SelectionListChanged);
                        this.canvas.Model.EventSink.PropertyChanged += new Syncfusion.Windows.Forms.Diagram.PropertyChangedEventHandler(EventSink_PropertyChanged);
                        this.canvas.Model.EventSink.PinPointChanged += new PinPointChangedEventHandler(EventSink_PinPointChanged);
                        
                        //this.canvas.SelectionChanged += new NodeCollectionEventHandler(canvas_SelectionChanged);
                        //this.canvas.PropertyChanged += new PropertyEventHandler(canvas_PropertyChanged);
                        //this.canvas.NodeMoved += new MoveEventHandler(canvas_NodeMoved);

						if (!this.DesignMode)
						{
							this.SetSelectedObject(this.canvas.Model);
						}
					}
				}
			}
		}

        void EventSink_PinPointChanged(PinPointChangedEventArgs evtArgs)
        {
            if (!this.DesignMode)
            {
                this.propGrid.Refresh();
            } 
        }

        void EventSink_PropertyChanged(Syncfusion.Windows.Forms.Diagram.PropertyChangedEventArgs evtArgs)
        {
            if (!this.DesignMode)
            {
                this.propGrid.Refresh();
            } 
        }

        void EventSink_SelectionListChanged(CollectionExEventArgs evtArgs)
        {
            if (evtArgs.Element != null)
            {
                Node node = evtArgs.Element as Node;
                if (!this.DesignMode)
                {
                    if (node == null || evtArgs.Elements.Count == 0)
                    {
                        this.propGrid.SelectedObject = this.canvas.View;
                        return;
                    }

                    //this.propGrid.SelectedObjects = null;
                    object[] selectedObjects = new object[canvas.View.SelectionList.Count];
                    for (int i = 0; i < canvas.View.SelectionList.Count; i++)
                    {
                        if (canvas.View.SelectionList[i].GetType().BaseType == typeof(Node))
                        {
                            Type nodeType = canvas.View.SelectionList[i].GetType();
                            Node symbol = canvas.View.SelectionList[i] as Node;
                            ControlNode ctrlNode = symbol as ControlNode;

                            selectedObjects[i] = ctrlNode.HostingControl.GetType().GetProperty("PT").GetValue(ctrlNode.HostingControl, null); //symbol.GetPropertyValue("PT");//nodeType.GetProperty("PT").GetValue(canvas.SelectionList[i], null);
                            //	this.propGrid.SelectedObject = nodeType.GetProperty("PT").GetValue(evtArgs.Nodes[0], null);;
                        }
                        else
                        {
                            Type nodeType = canvas.View.SelectionList[i].GetType();
                            selectedObjects[i] = canvas.View.SelectionList[i];
                        }
                    }
                    this.propGrid.SelectedObjects = selectedObjects;
                }
            }
        }

		/// <summary>
		/// Handles the SelectionChanged event of the canvas control.
		/// </summary>
		/// <param name="sender">The source of the event.</param>
		/// <param name="evtArgs">The <see cref="Syncfusion.Windows.Forms.Diagram.NodeCollectionEventArgs"/> instance containing the event data.</param>
        //private void canvas_SelectionChanged(object sender, Syncfusion.Windows.Forms.Diagram.NodeCollection evtArgs)
        //{
        //    if (!this.DesignMode)
        //    {
        //        if (evtArgs.Nodes == null || evtArgs.Nodes.Length == 0 || evtArgs.Nodes[0] == null)
        //        {
        //            this.propGrid.SelectedObject = this.canvas.View;
        //            return;
        //        }

        //        //this.propGrid.SelectedObjects = null;
        //        object[] selectedObjects = new object[canvas.SelectionList.Count];
        //        for(int i = 0; i < canvas.SelectionList.Count;i++)
        //        {
        //            if(canvas.SelectionList[i].GetType().BaseType == typeof(SymbolBase))
        //            {
        //                Type nodeType = canvas.SelectionList[i].GetType();
        //                Symbol symbol = canvas.SelectionList[i] as Symbol;
        //                ControlNode ctrlNode = symbol.Nodes.First as ControlNode;

        //                selectedObjects[i] = ctrlNode.Control.GetType().GetProperty("PT").GetValue(ctrlNode.Control, null); //symbol.GetPropertyValue("PT");//nodeType.GetProperty("PT").GetValue(canvas.SelectionList[i], null);
        //                //	this.propGrid.SelectedObject = nodeType.GetProperty("PT").GetValue(evtArgs.Nodes[0], null);;
        //            }
        //            else 
        //            {
        //                Type nodeType = canvas.SelectionList[i].GetType();
        //                selectedObjects[i] = canvas.SelectionList[i];
        //            }
        //        }
        //        this.propGrid.SelectedObjects = selectedObjects;
        //    }
        //}

		/// <summary>
		/// Handles the PropertyChanged event of the canvas control.
		/// </summary>
		/// <param name="sender">The source of the event.</param>
		/// <param name="evtArgs">The <see cref="Syncfusion.Windows.Forms.Diagram.PropertyEventArgs"/> instance containing the event data.</param>
        //private void canvas_PropertyChanged(object sender, Syncfusion.Windows.Forms.Diagram.PropertyChangedEventArgs evtArgs)
        //{
        //    if (!this.DesignMode)
        //    {
        //        //this.propGrid.Refresh(); 
        //    }
        //}

		/// <summary>
		/// Handles the NodeMoved event of the canvas control.
		/// </summary>
		/// <param name="sender">The source of the event.</param>
		/// <param name="evtArgs">The <see cref="Syncfusion.Windows.Forms.Diagram.MoveEventArgs"/> instance containing the event data.</param>
        //private void canvas_NodeMoved(object sender, MoveEventArgs eventargs)
        //{
        //    if (!this.DesignMode)
        //    {
        //        this.propGrid.Refresh();
        //    }                       
        //}

		/// <summary>
		/// Initializes the component.
		/// </summary>
		private void InitializeComponent()
		{
			this.components = new System.ComponentModel.Container();
			System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(PropertyEditor));
		}

		/// <summary>
		/// Handles the ValueChanged event of the PropertyGrid control.
		/// </summary>
		/// <param name="sender">The source of the event.</param>
		/// <param name="evtArgs">The <see cref="System.Windows.Forms.PropertyValueChangedEventArgs"/> instance containing the event data.</param>
		private void PropertyGrid_ValueChanged(object sender, PropertyValueChangedEventArgs evtArgs)
		{
			if (this.canvas != null)
			{
				//this.canvas.View.Update();
			}
		}

		/// <summary>
		/// Handles the Disposed event of the canvas control.
		/// </summary>
		/// <param name="sender">The source of the event.</param>
		/// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
		private void canvas_Disposed(object sender, EventArgs e)
		{
			//this.propGrid.SelectedObject = null;
		}
	}
}